#! /usr/bin/env python
#
# compat.py
#
# Version 3.0
#
# a program to quickly check for supported incompatibilities between tree files
#
# (c) F. Kauff / F. Lutzoni
#
# contact: fkauff@biologie.uni-kl.de
#

from Bio.Nexus import Nexus
from optparse import OptionParser
import sys,os,glob,sets

class CompatError(Exception):
    pass
    
class CompTrees:
    """Provides functionality for tree storage and comparision."""
    
    def __init__(self,trees,treefiles=None):
        self.trees=trees
        # filenames for the tree instances are optional
        self.treefiles=treefiles

    def add(self,trees,treefiles=None):
        """Add more trees for comparison."""
        if isinstance(trees,list):
            if treefiles is None:
                self.trees.extend(trees)
            else:
                if isinstance(treefiles,list) and len(trees)==len(treefiles):
                    self.treefiles.extend(treefiles)
                else:
                    raise CompatError, 'Tree(s) and Treefile(s) arguments do not match.'
        else:
            self.trees.append(trees)
            if treefiles is not None:
                self.trees.append(treefiles)

    def compall(self,threshold=0.7,strict=False):
        """Generator that does pairwise comparison of all trees in self.trees."""
        # trees, support and branchlengths come in many formats. If the tree has been
        # read with branchlengths only, but no support, we assume that support was
        # saved as branchlengths (as done by PAUP), and we convert branchlengths to support.
        # Otherwise probably everything is in place.
        for tn,tree in enumerate(self.trees):
            internal=[i for i in tree.chain.keys() if i not in tree.get_terminals()]
            for n in internal:
                if tree.node(n).data.support:
                    break
            else:   # no internal node has support
                tree.branchlength2support()
                self.trees[tn]=tree
        for i,t1 in enumerate(self.trees[:-1]):
            for j,t2 in enumerate(self.trees[i+1:]):
                compatibility=t1.is_compatible(t2,threshold,strict=strict)
                if self.treefiles:
                    yield t1.name,self.treefiles[i],t2.name,self.treefiles[i+j+1],compatibility
                else:
                    yield t1.name,None,t2.name,None,compatibility
                
def compare_trees(trees,threshold=0.7):
    compat_trees=CompTrees(trees)
    return [c for c in compat_trees.compall(threshold=threshold)]

# main
if __name__=='__main__':
    parser=OptionParser('usage: %prog [options] [treefile1 treefile2, ...]')
    parser.add_option('-t','--threshold',dest='threshold',help='support threshold',default=.95,type='float',metavar='<limit>')
    parser.add_option('-s','--strict',dest='strict',help='forces taxon sets in all trees to be identical',default=False,action='store_true')
    parser.add_option('-o','--outgroup',dest='outgroup',help='select common outgroup taxon to root trees prior to comparison',default=None,metavar='<taxon>',type='string')
    (options,args)=parser.parse_args(sys.argv[1:])
    treefiles=[]
    if len(args)<2:
        sys.exit('Please provide at least two tree files')
    for a in args:
        treefiles.extend(glob.glob(os.path.expanduser(a)))
    # some moron could submit the same file multiple times...
    treefiles.sort()
    #print treefiles
    if treefiles:
        treefiles=[f for i,f in enumerate(treefiles[:-1]) if f!=treefiles[i+1]]+[treefiles[-1]]
    if len(treefiles)<2:
        sys.exit('At least two trees are needed for comparison.')
    try:
        trees=[Nexus.Nexus(t).trees[0] for t in treefiles]
    except:
        raise
        sys.exit('Could not read tree from file: '+t)
    if options.outgroup:
        for t in trees:
            t.root_with_outgroup(outgroup=options.outgroup)
    compat_trees=CompTrees(trees,treefiles)
    for tree1,file1,tree2,file2,conflict in compat_trees.compall(threshold=options.threshold,strict=options.strict):
        if conflict:
            #print '-'*10
            print '\nConflicts found: %s (%s) <--> %s (%s)' % (tree1,file1,tree2,file2)
            for c in conflict:
                print 'Tree %s, clade support = %2.2f:' % (tree1,c[1]),
                print ', '.join(c[0])
                print 'Tree %s, clade support = %2.2f:' % (tree2,c[3]),
                print ', '.join(c[2])
                print 'Shared taxa in both clades: %s' % (', '.join(list(c[4])))
                print 'Taxa not present in clade of tree %s: %s' % (tree1,', '.join(list(c[5])))
                print 'Taxa not present in clade of tree %s: %s' % (tree2,', '.join(list(c[6])))
                print

                
    
        




